if select(2, UnitClass("player")) ~= "MONK" then return end

local AceConfig = LibStub("AceConfig-3.0") -- For the options panel
local AceConfigDialog = LibStub("AceConfigDialog-3.0") -- Also for options panel
local AceDB = LibStub("AceDB-3.0") -- Makes saving things really easy
local AceDBOptions = LibStub("AceDBOptions-3.0") -- More database options
local AceConfigCmd =LibStub("AceConfigCmd-3.0")
--------------------------------------------------------------------------------------
local AceLocale = LibStub("AceLocale-3.0")
local L = AceLocale:GetLocale( "Transcendence" )
local LSM = LibStub("LibSharedMedia-3.0")
local string_match = string.match
local math_sqrt = math.sqrt
local max = max
local min = min
local GetSpellInfo = GetSpellInfo
--------------------------------------------------------------------------------------

local cvar_width,cvar_height = string_match(GetCVar("gxResolution"), "(%d+)x(%d+)")
cvar_width = tonumber(cvar_width)
cvar_height = tonumber(cvar_height)
local cvar_min = min(cvar_width,cvar_height)
--------------------------------------------------------------------------------------
local order = 0
local function get_order()
	local temp = order
	order = order +1
	return temp
end

Transcendence.options =
{
	type = "group",
	order = get_order(),
	name = "Transcendence",
	args =
	{
		lock =
		{
			order = get_order(),
			name = L["Lock"],
			desc = L["Locks the positions of the frames"],
			type = "toggle",
			set = function(info,val)
				Transcendence.db.profile.lock = val
				Transcendence:lock()
			end,
			get = function(info) return Transcendence.db.profile.lock end,
			width = "full"
		},
		refreshRate =
		{
			type = "range",
			order = get_order(),
			name = L["Refresh Rate"],
			desc = L["How often the frame will be updated. less means more updates per second at a cost of performance."],
			min = 0,
			max = 2,
			step = 0.01,
			get = function () return Transcendence.db.profile.refreshRate end,
			set = function (_, v)
				Transcendence.db.profile.refreshRate = v
				Transcendence:refresh_rate_modified()
			end,
			width = "full"
		},
		Icon =
		{
			order = get_order(),
			name = L["Icon"],
			type = "group",
			args =
			{
				LeftPos = 
				{
					type = "range",
					order = get_order(),
					name = L["Left Position"],
					min = 0,
					max = cvar_width,
					step = 0.1,
					get = function () return Transcendence.db.profile.pos_left end,
					set = function (_, v)
						Transcendence.db.profile.pos_left = v
						Transcendence:refresh_location()
					end
				},
				TopPos = 
				{
					type = "range",
					order = get_order(),
					name = L["Top Position"],
					min = 0,
					max = cvar_height,
					step = 0.1,
					get = function () return Transcendence.db.profile.pos_top end,
					set = function (_, v)
						Transcendence.db.profile.pos_top = v
						Transcendence:refresh_location()
					end
				},
				iconSize =
				{
					type = "range",
					order = get_order(),
					name = L["Size"],
					min = 10,
					max = cvar_min,
					step = 0.1,
					get = function () return Transcendence.db.profile.iconSize end,
					set = function (_, v)
						Transcendence.db.profile.iconSize = v
						Transcendence:refresh_icon_size()
					end
				},
				arrowColor =
				{
					type = "color",
					order = get_order(),
					name = L["Arrow Color"],
					hasAlpha = true,
					get = function ()return Transcendence.db.profile.arrowColorr,Transcendence.db.profile.arrowColorg,Transcendence.db.profile.arrowColorb,Transcendence.db.profile.arrowColora end,
					set = function (_, r,g,b,a)
						Transcendence.db.profile.arrowColorr,Transcendence.db.profile.arrowColorg,Transcendence.db.profile.arrowColorb,Transcendence.db.profile.arrowColora = r,g,b,a
						Transcendence:refresh_icon_size()
					end,
				},
			}
		},
		Image =
		{
			order = get_order(),
			name = L["Image"],
			type = "group",
			args =
			{
				TransferImage =
				{
					type = "input",
					order = get_order(),
					name = L["Transfer"],
					desc = L["Warning! Please make sure the image exists"],
					multiline = true,
					get = function ()return Transcendence.db.profile.transferImage end,
					set = function (_, v)
						Transcendence.db.profile.transferImage = v
						Transcendence:refresh_Transfer_Image()
					end,
					width = "full"
				},
				UnknownTransferImage =
				{
					type = "input",
					order = get_order(),
					name = L["Unknown Transfer"],
					desc = L["Warning! Please make sure the image exists"],
					multiline = true,
					get = function ()return Transcendence.db.profile.unknownTransferImage end,
					set = function (_, v)
						Transcendence.db.profile.unknownTransferImage = v
						Transcendence:refresh_Unknown_Transfer_Image()
					end,
					width = "full"
				},
				NoTransferImage =
				{
					type = "input",
					order = get_order(),
					name = L["No Transfer"],
					desc = L["Warning! Please make sure the image exists"],
					multiline = true,
					get = function ()return Transcendence.db.profile.noTransferImage end,
					set = function (_, v)
						Transcendence.db.profile.noTransferImage = v
						Transcendence:refresh_No_Transfer_Image()
					end,
					width = "full"
				},
			}
		},
		Distance = 
		{
			order = get_order(),
			name = L["Distance"],
			type = "group",
			args =
			{
				disFont =
				{
					type = 'select',
					dialogControl = 'LSM30_Font',
					name = L["Font"],
					values = LSM:HashTable("font"),
					order = get_order(),
					get = function()
						return Transcendence.db.profile.disFont
					end,
					set = function(_,v)
						Transcendence.db.profile.disFont = v
						Transcendence:refresh_font_size()
					end,
					width = "full"
				},
				fontSize =
				{
					type = "range",
					order = get_order(),
					name = L["Font Size"],
					min = 0.1,
					max = 100,
					step = 0.1,
					get = function () return Transcendence.db.profile.fontSize end,
					set = function (_, v)
						Transcendence.db.profile.fontSize = v
						Transcendence:refresh_font_size()
					end,
					width = "full"
				},
				disFixedPrecision =
				{
					type = "range",
					order = get_order(),
					name = L["Fixed Precision"],
					min = 0,
					max = 15,
					step = 1,
					get = function () return Transcendence.db.profile.disFixedPrecision end,
					set = function (_, v)
						Transcendence.db.profile.disFixedPrecision = v
						Transcendence:refresh_dis_fixed_precision()
					end,
					width = "full"
				},
				shortDistanceColor =
				{
					type = "color",
					order = get_order(),
					name = L["Short Distance Font Color"],
					hasAlpha = true,
					get = function ()return Transcendence.db.profile.shortDistanceColorr,Transcendence.db.profile.shortDistanceColorg,Transcendence.db.profile.shortDistanceColorb,Transcendence.db.profile.shortDistanceColora end,
					set = function (_, r,g,b,a)
						Transcendence.db.profile.shortDistanceColorr,Transcendence.db.profile.shortDistanceColorg,Transcendence.db.profile.shortDistanceColorb,Transcendence.db.profile.shortDistanceColora = r,g,b,a
						Transcendence:refresh_SHORT_DISTANCE_COLOR()
					end,
				},
				middleDistanceColor =
				{
					type = "color",
					order = get_order(),
					name = L["Middle Distance Font Color"],
					hasAlpha = true,
					get = function ()return Transcendence.db.profile.middleDistanceColorr,Transcendence.db.profile.middleDistanceColorg,Transcendence.db.profile.middleDistanceColorb,Transcendence.db.profile.middleDistanceColora end,
					set = function (_, r,g,b,a)
						Transcendence.db.profile.middleDistanceColorr,Transcendence.db.profile.middleDistanceColorg,Transcendence.db.profile.middleDistanceColorb,Transcendence.db.profile.middleDistanceColora = r,g,b,a
						Transcendence:refresh_MIDDLE_DISTANCE_COLOR()
					end,
				},
				longDistanceColor =
				{
					type = "color",
					order = get_order(),
					name = L["Long Distance Font Color"],
					hasAlpha = true,
					get = function ()return Transcendence.db.profile.longDistanceColorr,Transcendence.db.profile.longDistanceColorg,Transcendence.db.profile.longDistanceColorb,Transcendence.db.profile.longDistanceColora end,
					set = function (_, r,g,b,a)
						Transcendence.db.profile.longDistanceColorr,Transcendence.db.profile.longDistanceColorg,Transcendence.db.profile.longDistanceColorb,Transcendence.db.profile.longDistanceColora = r,g,b,a
						Transcendence:refresh_LONG_DISTANCE_COLOR()
					end,
				},
			}
		},
		Cooldown = 
		{
			order = get_order(),
			name = L["Cooldown"],
			type = "group",
			args =
			{
				enableCD =
				{
					order = get_order(),
					name = L["Enable"],
					type = "toggle",
					set = function(info,val)
						Transcendence.db.profile.enableCD = val
						Transcendence:UpdateDB()
					end,
					get = function(info) return Transcendence.db.profile.enableCD end,
					width = "full"
				},
				cdFont =
				{
					type = 'select',
					dialogControl = 'LSM30_Font',
					name = L["Font"],
					values = LSM:HashTable("font"),
					get = function()
						return Transcendence.db.profile.cdFont
					end,
					set = function(_,v)
						Transcendence.db.profile.cdFont = v
						Transcendence:refresh_cd_font_size()
					end,
				},
				cdfontSize =
				{
					type = "range",
					order = get_order(),
					name = L["Font Size"],
					min = 0.1,
					max = 100,
					step = 0.1,
					get = function () return Transcendence.db.profile.cdFontSize end,
					set = function (_, v)
						Transcendence.db.profile.cdFontSize = v
						Transcendence:refresh_cd_font_size()
					end,
				},
				cdFixedPrecision =
				{
					type = "range",
					order = get_order(),
					name = L["Fixed Precision"],
					min = 0,
					max = 15,
					step = 1,
					get = function () return Transcendence.db.profile.cdFixedPrecision end,
					set = function (_, v)
						Transcendence.db.profile.cdFixedPrecision = v
						Transcendence:refresh_cd_fixed_precision()
					end,
				},
				cdReadyFontColor =
				{
					type = "color",
					order = get_order(),
					name = L["Ready font color"],
					hasAlpha = true,
					get = function ()return Transcendence.db.profile.cdReadyFontr,Transcendence.db.profile.cdReadyFontg,Transcendence.db.profile.cdReadyFontb,Transcendence.db.profile.cdReadyFonta end,
					set = function (_, r,g,b,a)
						Transcendence.db.profile.cdReadyFontr,Transcendence.db.profile.cdReadyFontg,Transcendence.db.profile.cdReadyFontb,Transcendence.db.profile.cdReadyFonta = r,g,b,a
						Transcendence:refresh_cdReadyFontColor()
					end,
				},
				cdFontColor =
				{
					type = "color",
					order = get_order(),
					name = L["Font color"],
					hasAlpha = true,
					get = function ()return Transcendence.db.profile.cdFontColorr,Transcendence.db.profile.cdFontColorg,Transcendence.db.profile.cdFontColorb,Transcendence.db.profile.cdFontColora end,
					set = function (_, r,g,b,a)
						Transcendence.db.profile.cdFontColorr,Transcendence.db.profile.cdFontColorg,Transcendence.db.profile.cdFontColorb,Transcendence.db.profile.cdFontColora = r,g,b,a
						Transcendence:refresh_cdFontColor()
					end,
				}
			}
		},
		Calculation =
		{
			order = get_order(),
			name = L["Calculation"],
			type = "group",
			args =
			{
				mapErrorFactor =
				{
					type = "range",
					order = get_order(),
					name = L["Map Error Factor"],
					min = 0,
					max = 1,
					step = 0.01,
					get = function () return Transcendence.db.profile.mapErrorFactor end,
					set = function (_, v)
						Transcendence.db.profile.mapErrorFactor = v
						Transcendence:refresh_TRANSFER_DISTANCE()
					end,
					width = "full"
				},
				ReadyCD =
				{
					type = "range",
					order = get_order(),
					name = L["Seconds Before Ready"],
					min = 0,
					max = 20,
					step = 0.01,
					get = function () return Transcendence.db.profile.readyCD end,
					set = function (_, v)
						Transcendence.db.profile.readyCD = v
						Transcendence:refresh_ReadyCD()
					end,
					width = "full"
				},
				longDistance =
				{
					type = "range",
					order = get_order(),
					name = L["Long Distance"],
					min = 40,
					max = 1000,
					step = 0.01,
					get = function () return Transcendence.db.profile.longDistance end,
					set = function (_, v)
						Transcendence.db.profile.longDistance = v
						Transcendence:refresh_LONG_DISTANCE()
					end,
					width = "full"
				},
			},
		},
	}
}
Transcendence.default_options=
{
	profile = 
	{
		lock = false,
		pos_left = cvar_width / 3 * 2,
		pos_top = cvar_height / 2,
		mapErrorFactor = 0.95,
		refreshRate = 0.1,
		fontSize = 30,
		cdFontSize = 15,
		iconSize = 65,
		disFixedPrecision = 0,
		cdFixedPrecision = 1,
		arrowColorr = 1,
		arrowColorg = 1,
		arrowColorb = 1,
		arrowColora = 1,
		enableCD = true,
		longDistance = 80,
		disFont = LSM:GetDefault("font"),
		cdFont = LSM:GetDefault("font"),
		transferImage = "Interface\\Icons\\monk_stance_redcrane",
		unknownTransferImage = "Interface\\Icons\\ability_monk_roll",
		noTransferImage = "Interface\\Icons\\monk_ability_transcendence",
		readyCD = 3,
		cdReadyFontr= 1,
		cdReadyFontg= 0,
		cdReadyFontb= 0,
		cdReadyFonta= 1,
		cdFontColorr= 1,
		cdFontColorg= 1,
		cdFontColorb= 1,
		cdFontColora= 1,
		shortDistanceColorr = 0,
		shortDistanceColorg = 1,
		shortDistanceColorb = 0,
		shortDistanceColora = 1,
		middleDistanceColorr = 1,
		middleDistanceColorg = 1,
		middleDistanceColorb = 0,
		middleDistanceColora = 1,
		longDistanceColorr = 1,
		longDistanceColorg = 1,
		longDistanceColorb = 0,
		longDistanceColora = 1,
		longDistanceColorr = 1,
		longDistanceColorg = 0,
		longDistanceColorb = 0,
		longDistanceColora = 1,
	}
}
function Transcendence:UpdateDB()
	Transcendence:lock()
	Transcendence:refresh_location()
	Transcendence:refresh_TRANSFER_DISTANCE()
	Transcendence:refresh_rate_modified()
	Transcendence:refresh_font_size()
	Transcendence:refresh_cd_font_size()
	Transcendence:refresh_icon_size()
	Transcendence:refresh_dis_fixed_precision()
	Transcendence:refresh_enable_cd()
	Transcendence:refresh_cd_fixed_precision()
	Transcendence:refresh_LONG_DISTANCE()
	Transcendence:refresh_SHORT_DISTANCE_COLOR()
	Transcendence:refresh_MIDDLE_DISTANCE_COLOR()
	Transcendence:refresh_LONG_DISTANCE_COLOR()
	Transcendence:refresh_Transfer_Image()
	Transcendence:refresh_Unknown_Transfer_Image()
	Transcendence:refresh_No_Transfer_Image()
	Transcendence:refresh_ReadyCD()
	Transcendence:refresh_cdReadyFontColor()
	Transcendence:refresh_cdFontColor()
end
function Transcendence:OnInitialize()
	Transcendence.db = AceDB:New("TranscendenceDB", Transcendence.default_options,true)
	Transcendence.options.args.profile = AceDBOptions:GetOptionsTable(Transcendence.db)
	AceConfig:RegisterOptionsTable("Transcendence", Transcendence.options, nil)
	Transcendence.optionsFrames = {}
	Transcendence.optionsFrames.general = AceConfigDialog:AddToBlizOptions("Transcendence", "Transcendence")
	self:Init()
	self.db.RegisterCallback(self, "OnProfileChanged", "UpdateDB")
	self.db.RegisterCallback(self, "OnProfileCopied", "UpdateDB")
	self.db.RegisterCallback(self, "OnProfileReset", "UpdateDB")
	self:RegisterChatCommand("Transcendence", "ChatCommand")
	self:RegisterChatCommand("TRD", "ChatCommand")
end
function Transcendence:OnEnable()
	self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
	self:RegisterEvent("UNIT_SPELLCAST_SUCCEEDED")
	self:RegisterEvent("UNIT_SPELLCAST_SENT")
	self:RegisterEvent("ZONE_CHANGED_NEW_AREA")
end
function Transcendence:ChatCommand(input)
	if not input or input:trim() == "" then
		AceConfigDialog:Open("Transcendence")
	else
		AceConfigCmd:HandleCommand("Transcendence", "Transcendence","")
		AceConfigCmd:HandleCommand("Transcendence", "Transcendence",input)
	end
end
